@@ -10,7 +10,7 @@ package Affix {    # 'FFI' is my middle name!
     use Text::ParseWords;
     use Carp qw[];
     use vars qw[@EXPORT_OK @EXPORT %EXPORT_TAGS];
-    our $VERSION = '0.09_06';
+    our $VERSION = '0.10_01';
     use XSLoader;
     XSLoader::load( __PACKAGE__, $VERSION );
     #
@@ -265,12 +265,6 @@ package Affix {    # 'FFI' is my middle name!
         return $_lib_cache->{ $name . ';' . ( $version // '' ) }
             // Carp::croak( 'Cannot locate symbol: ' . $name );
     }
-
-    # define packages that are otherwise XS-only so PAUSE will find them in META.json
-    {
-
-        package Affix::Feature 0.09;
-    }
 };
 1;
 __END__
@@ -1076,6 +1070,25 @@ declaration is straightforward:
 
     TODO
 
+=head1 Features
+
+Not all features of dyncall are supported on all platforms, for those, the
+underlying library defines macros you can use to detect support. These values
+are exposed under the C<Affix::Feature> package:
+
+=over
+
+=item C<Affix::Feature::Syscall()>
+
+If true, your platform supports a syscall calling conventions.
+
+=item C<Affix::Feature::AggrByVal()>
+
+If true, your platform supports passing around aggregates (struct, union) by
+value.
+
+=back
+
 =head1 See Also
 
 Check out L<FFI::Platypus> for a more robust and mature FFI.
@@ -1102,7 +1115,7 @@ Sanko Robinson E<lt>sanko@cpan.orgE<gt>
 
 dyncall OpenBSD FreeBSD macOS DragonFlyBSD NetBSD iOS ReactOS mips mips64 ppc32
 ppc64 sparc sparc64 co-existing varargs variadic struct enum eXtension rvalue
-dualvars libsomething versioned errno
+dualvars libsomething versioned errno syscall
 
 =end stopwords
 
