@@ -23,7 +23,7 @@ extern "C" {
 #define dcAllocMem safemalloc
 #define dcFreeMem Safefree
 
-//#include "ppport.h"
+// #include "ppport.h"
 
 #ifndef av_count
 #define av_count(av) (AvFILL(av) + 1)
@@ -51,15 +51,15 @@ extern "C" {
 #include <dyncall/dyncall/dyncall_aggregate.h>
 
 //{ii[5]Z&<iZ>}
-#define DC_SIGCHAR_CODE '&'      // 'p' but allows us to wrap CV * for the user
-#define DC_SIGCHAR_ARRAY '['     // 'A' but nicer
-#define DC_SIGCHAR_STRUCT '{'    // 'A' but nicer
-#define DC_SIGCHAR_UNION '<'     // 'A' but nicer
-#define DC_SIGCHAR_BLESSED '$'   // 'p' but an object or subclass of a given package
-#define DC_SIGCHAR_ANY '*'       // 'p' but it's really an SV/HV/AV
-#define DC_SIGCHAR_ENUM 'e'      // 'i' but with multiple options
-#define DC_SIGCHAR_ENUM_UINT 'E' // 'I' but with multiple options
-#define DC_SIGCHAR_ENUM_CHAR 'o' // 'c' but with multiple options
+#define DC_SIGCHAR_CODE '&'       // 'p' but allows us to wrap CV * for the user
+#define DC_SIGCHAR_ARRAY '['      // 'A' but nicer
+#define DC_SIGCHAR_STRUCT '{'     // 'A' but nicer
+#define DC_SIGCHAR_UNION '<'      // 'A' but nicer
+#define DC_SIGCHAR_INSTANCEOF '$' // 'p' but an object or subclass of a given package
+#define DC_SIGCHAR_ANY '*'        // 'p' but it's really an SV/HV/AV
+#define DC_SIGCHAR_ENUM 'e'       // 'i' but with multiple options
+#define DC_SIGCHAR_ENUM_UINT 'E'  // 'I' but with multiple options
+#define DC_SIGCHAR_ENUM_CHAR 'o'  // 'c' but with multiple options
 
 // MEM_ALIGNBYTES is messed up by quadmath and long doubles
 #define AFFIX_ALIGNBYTES 8
@@ -192,6 +192,7 @@ SV *ptr2sv(pTHX_ DCpointer ptr, SV *type);
 /* Returns the amount of padding needed after `offset` to ensure that the
 following address will be aligned to `alignment`. */
 size_t padding_needed_for(size_t offset, size_t alignment) {
+    if (alignment == 0) return 0;
     size_t misalignment = offset % alignment;
     if (misalignment) // round to the next multiple of alignment
         return alignment - misalignment;
@@ -309,10 +310,13 @@ char cbHandler(DCCallback *cb, DCArgs *args, DCValue *result, DCpointer userdata
 
     ENTER;
     SAVETMPS;
+
     PUSHMARK(SP);
 
+    EXTEND(SP, cbx->sig_len);
+
     if (cbx->sig_len) {
-        EXTEND(SP, cbx->sig_len);
+
         char type;
         for (size_t i = 0; i < cbx->sig_len; ++i) {
             type = cbx->sig[i];
@@ -366,13 +370,13 @@ char cbHandler(DCCallback *cb, DCArgs *args, DCValue *result, DCpointer userdata
                 break;
             case DC_SIGCHAR_POINTER: {
                 DCpointer ptr = dcbArgPointer(args);
+                //~ warn("here %p at %s line %d", ptr, __FILE__, __LINE__);
+
                 SV *__type = *av_fetch(cbx->args, i, 0);
                 char *_type = SvPV_nolen(__type);
-                // warn("_type == %s", _type);
                 switch (_type[0]) { // true type
                 case DC_SIGCHAR_ANY: {
-                    SV *s = ptr2sv(aTHX_ ptr, (__type));
-                    sv_dump(s);
+                    SV *s = ptr2sv(aTHX_ ptr, __type);
                     mPUSHs(s);
                 } break;
                 case DC_SIGCHAR_CODE: {
@@ -388,7 +392,7 @@ char cbHandler(DCCallback *cb, DCArgs *args, DCValue *result, DCpointer userdata
                 DCpointer ptr = dcbArgPointer(args);
                 PUSHs(newSVpv((char *)ptr, 0));
             } break;
-            case DC_SIGCHAR_BLESSED: {
+            case DC_SIGCHAR_INSTANCEOF: {
                 DCpointer ptr = dcbArgPointer(args);
                 HV *blessed = MUTABLE_HV(SvRV(*av_fetch(cbx->args, i, 0)));
                 SV **package = hv_fetchs(blessed, "package", 0);
@@ -411,63 +415,68 @@ char cbHandler(DCCallback *cb, DCArgs *args, DCValue *result, DCpointer userdata
                 croak("Unhandled callback arg. Type: %c [%s]", cbx->sig[i], cbx->sig);
                 break;
             }
-        }  }
-        PUTBACK;
+        }
+    }
+    PUTBACK;
 
-    warn("here at %s line %d", __FILE__, __LINE__);
-    if (cbx->ret == DC_SIGCHAR_VOID) { call_sv(cbx->cv, G_VOID); }
-    else {
-        warn("here at %s line %d", __FILE__, __LINE__);
-        count = call_sv(cbx->cv, G_SCALAR);
-        if (count != 1) croak("Big trouble: %d returned items", count);
+    // warn("cbx->ret eq '%c' at %s line %d", cbx->ret, __FILE__, __LINE__);
+    if (cbx->ret == DC_SIGCHAR_VOID) {
+        count = call_sv(cbx->cv, G_VOID);
         SPAGAIN;
+    }
+    else {
+        count = call_sv(cbx->cv, G_SCALAR);
+        SPAGAIN;
+        if (count != 1) croak("Big trouble: %d returned items", count);
+
+        SV *ret = POPs;
+
         switch (cbx->ret) {
         case DC_SIGCHAR_VOID:
             break;
         case DC_SIGCHAR_BOOL:
-            result->B = SvTRUEx(POPs);
+            result->B = SvTRUEx(ret);
             break;
         case DC_SIGCHAR_CHAR:
-            result->c = POPu;
+            result->c = SvIOK(ret) ? SvIV(ret) : 0;
             break;
         case DC_SIGCHAR_UCHAR:
-            result->C = POPu;
+            result->C = SvIOK(ret) ? ((UV)SvUVx(ret)) : 0;
             break;
         case DC_SIGCHAR_SHORT:
-            result->s = POPu;
+            result->s = SvIOK(ret) ? SvIVx(ret) : 0;
             break;
         case DC_SIGCHAR_USHORT:
-            result->S = POPi;
+            result->S = SvIOK(ret) ? SvUVx(ret) : 0;
             break;
         case DC_SIGCHAR_INT:
-            result->i = POPi;
+            result->i = SvIOK(ret) ? SvIVx(ret) : 0;
             break;
         case DC_SIGCHAR_UINT:
-            result->I = POPu;
+            result->I = SvIOK(ret) ? SvUVx(ret) : 0;
             break;
         case DC_SIGCHAR_LONG:
-            result->j = POPl;
+            result->j = SvIOK(ret) ? SvIVx(ret) : 0;
             break;
         case DC_SIGCHAR_ULONG:
-            result->J = POPul;
+            result->J = SvIOK(ret) ? SvUVx(ret) : 0;
             break;
         case DC_SIGCHAR_LONGLONG:
-            result->l = POPi;
+            result->l = SvIOK(ret) ? SvIVx(ret) : 0;
             break;
         case DC_SIGCHAR_ULONGLONG:
-            result->L = POPu;
+            result->L = SvIOK(ret) ? SvUVx(ret) : 0;
             break;
         case DC_SIGCHAR_FLOAT:
-            result->f = POPn;
+            result->f = SvNOK(ret) ? SvNVx(ret) : 0.0;
             break;
         case DC_SIGCHAR_DOUBLE:
-            result->d = POPn;
+            result->d = SvNOK(ret) ? SvNVx(ret) : 0.0;
             break;
         case DC_SIGCHAR_POINTER: {
-            SV *sv_ptr = POPs;
-            if (SvOK(sv_ptr)) {
-                if (sv_derived_from(sv_ptr, "Affix::Pointer")) {
-                    IV tmp = SvIV((SV *)SvRV(sv_ptr));
+            if (SvOK(ret)) {
+                if (sv_derived_from(ret, "Affix::Pointer")) {
+                    IV tmp = SvIV((SV *)SvRV(ret));
                     result->p = INT2PTR(DCpointer, tmp);
                 }
                 else
@@ -477,13 +486,16 @@ char cbHandler(DCCallback *cb, DCArgs *args, DCValue *result, DCpointer userdata
                 result->p = NULL; // ha.
         } break;
         case DC_SIGCHAR_STRING:
-            result->Z = POPp;
+            result->Z = SvPOK(ret) ? SvPVx_nolen_const(ret) : NULL;
             break;
+        case DC_SIGCHAR_STRUCT:
+        case DC_SIGCHAR_UNION:
+        case DC_SIGCHAR_INSTANCEOF:
         default:
             croak("Unhandled return from callback: %c", cbx->ret);
         }
-        PUTBACK;
     }
+    PUTBACK;
 
     FREETMPS;
     LEAVE;
@@ -519,6 +531,39 @@ bool is_valid_class_name(SV *sv) { // Stolen from Type::Tiny::XS::Util
     return RETVAL;
 }
 
+SV *_instanceof(pTHX_ SV *type) { // Lazy load actual type from typemap and InstanceOf[]
+    SV *retval;
+    char *name = SvPV_nolen(*hv_fetchs(MUTABLE_HV(SvRV(type)), "package", 0));
+    {
+        dSP;
+        int count;
+        SV *err_tmp;
+        ENTER;
+        SAVETMPS;
+
+        PUSHMARK(SP);
+        PUTBACK;
+
+        count = call_pv(name, G_SCALAR | G_EVAL);
+
+        SPAGAIN;
+
+        err_tmp = ERRSV;
+        if (SvTRUE(err_tmp)) {
+            croak("Malformed InstanceOf[ '%s' ]; %s\n", name, SvPV_nolen(err_tmp));
+            POPs;
+        }
+        else {
+            if (count != 1) croak("Malformed InstanceOf[ '%s' ]; missing typedef", name);
+            retval = POPs;
+            SvSetMagicSV(type, retval);
+        }
+        FREETMPS;
+        LEAVE;
+    }
+    return type;
+}
+
 static size_t _sizeof(pTHX_ SV *type) {
     char *_type =
         SvPVbytex_nolen(type); // stringify to sigchar; speed cheat vs sv_derived_from(...)
@@ -556,9 +601,10 @@ static size_t _sizeof(pTHX_ SV *type) {
     case DC_SIGCHAR_CODE: // automatically wrapped in a DCCallback pointer
     case DC_SIGCHAR_POINTER:
     case DC_SIGCHAR_STRING:
-    case DC_SIGCHAR_BLESSED:
     case DC_SIGCHAR_ANY:
         return PTRSIZE;
+    case DC_SIGCHAR_INSTANCEOF:
+        return _sizeof(aTHX_ _instanceof(aTHX_ type));
     default:
         croak("Failed to gather sizeof info for unknown type: %s", _type);
         return -1;
@@ -632,7 +678,8 @@ static DCaggr *_aggregate(pTHX_ SV *type) {
                 } break;
                 case DC_SIGCHAR_ANY:
                 case DC_SIGCHAR_CODE:
-                case DC_SIGCHAR_POINTER: {
+                case DC_SIGCHAR_POINTER:
+                case DC_SIGCHAR_INSTANCEOF: {
                     dcAggrField(agg, DC_SIGCHAR_POINTER, offset, 1);
                 } break;
                 default: {
@@ -751,6 +798,8 @@ SV *ptr2sv(pTHX_ DCpointer ptr, SV *type) {
         SvSetSV(RETVAL, newRV(MUTABLE_SV(RETVAL_)));
     } break;
     case DC_SIGCHAR_POINTER: {
+        // ptr = *(DCpointer *)ptr;
+        //~ warn("here %p at %s line %d", ptr, __FILE__, __LINE__);
         SV *subtype;
         if (sv_derived_from(type, "Affix::Type::Pointer"))
             subtype = *hv_fetchs(MUTABLE_HV(SvRV(type)), "type", 0);
@@ -758,11 +807,13 @@ SV *ptr2sv(pTHX_ DCpointer ptr, SV *type) {
             subtype = type;
         char *_subtype = SvPV_nolen(subtype);
         if (_subtype[0] == DC_SIGCHAR_VOID) {
-            SV *RETVALSV = newSV(0); // sv_newmortal();
-            SvSetSV(RETVAL, sv_setref_pv(RETVALSV, "Dyn::Call::Pointer", ptr));
+            //~ warn("void pointer [%p] at %s line %d", ptr, __FILE__, __LINE__);
+            SV *RETVALSV = newSV(1); // sv_newmortal();
+            SvSetSV(RETVAL, sv_setref_pv(RETVALSV, "Affix::Pointer", *(DCpointer *)ptr));
+            // sv_setref_pv(RETVAL, "Affix::Pointer", ptr);
         }
         else {
-            char *_subtype = SvPV_nolen(subtype);
+            //~ warn("some other pointer at %s line %d", __FILE__, __LINE__);
             SvSetSV(RETVAL, ptr2sv(aTHX_ ptr, subtype));
         }
     } break;
@@ -778,6 +829,18 @@ SV *ptr2sv(pTHX_ DCpointer ptr, SV *type) {
         SvSetSV(RETVAL, cb->cv);
         // warn("here at %s line %d", __FILE__, __LINE__);
     } break;
+    case DC_SIGCHAR_INSTANCEOF: {
+        RETVAL = ptr2sv(aTHX_ ptr, _instanceof(aTHX_ type));
+    } break;
+    case DC_SIGCHAR_ENUM: {
+        SvSetSV(RETVAL, enum2sv(aTHX_ type, *(int *)ptr));
+    }; break;
+    case DC_SIGCHAR_ENUM_UINT: {
+        SvSetSV(RETVAL, enum2sv(aTHX_ type, *(unsigned int *)ptr));
+    }; break;
+    case DC_SIGCHAR_ENUM_CHAR: {
+        SvSetSV(RETVAL, enum2sv(aTHX_ type, (IV) * (char *)ptr));
+    }; break;
     default:
         croak("Oh, this is unexpected: %c", _type[0]);
     }
@@ -789,13 +852,13 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
     // warn("sv2ptr( %s, data, %p, packed) at %s line %d", str, ptr, __FILE__, __LINE__);
     switch (str[0]) {
     case DC_SIGCHAR_VOID: {
-        if (sv_derived_from(data, "Dyn::Call::Pointer")) {
+        if (sv_derived_from(data, "Affix::Pointer")) {
             IV tmp = SvIV((SV *)SvRV(data));
             DCpointer ptr = INT2PTR(DCpointer, tmp);
             Copy((DCpointer)(&data), ptr, 1, intptr_t);
         }
         else
-            croak("Expected a subclass of Dyn::Call::Pointer");
+            croak("Expected a subclass of Affix::Pointer");
     } break;
     case DC_SIGCHAR_BOOL: {
         bool value = SvTRUE(data);
@@ -830,10 +893,12 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
         unsigned short value = (unsigned short)SvUV(data);
         Copy(&value, ptr, 1, unsigned short);
     } break;
+    case DC_SIGCHAR_ENUM:
     case DC_SIGCHAR_INT: {
         int value = SvIV(data);
         Copy(&value, ptr, 1, int);
     } break;
+    case DC_SIGCHAR_ENUM_UINT:
     case DC_SIGCHAR_UINT: {
         unsigned int value = SvUV(data);
         Copy(&value, ptr, 1, unsigned int);
@@ -859,14 +924,8 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
         Copy(&value, ptr, 1, float);
     } break;
     case DC_SIGCHAR_DOUBLE: {
-        warn("here at %s line %d", __FILE__, __LINE__);
-
         double value = SvNV(data);
-        warn("here at %s line %d", __FILE__, __LINE__);
-
         Copy(&value, ptr, 1, double);
-        warn("here at %s line %d", __FILE__, __LINE__);
-
     } break;
     case DC_SIGCHAR_STRING: {
         const char *str = SvPV_nolen(data);
@@ -882,6 +941,13 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
         sv2ptr(aTHX_ * type_ptr, data, value, packed);
         Copy(&value, ptr, 1, intptr_t);
     } break;
+    case DC_SIGCHAR_INSTANCEOF: {
+        HV *hv_ptr = MUTABLE_HV(SvRV(type));
+        SV **type_ptr = hv_fetchs(hv_ptr, "type", 0);
+        DCpointer value = safemalloc(_sizeof(aTHX_ * type_ptr));
+        sv2ptr(aTHX_ _instanceof(aTHX_ * type_ptr), data, value, packed);
+        Copy(&value, ptr, 1, intptr_t);
+    } break;
     case DC_SIGCHAR_STRUCT: {
         if (SvTYPE(SvRV(data)) != SVt_PVHV) croak("Expected a hash reference");
         size_t size = _sizeof(aTHX_ type);
@@ -977,12 +1043,12 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
 
         callback->args = MUTABLE_AV(SvRV(*args));
         callback->sig = SvPV_nolen(*sig);
-        callback->sig_len = SvIV(*sig_len);
+        callback->sig_len = (size_t)SvIV(*sig_len);
         callback->ret = (char)*SvPV_nolen(*ret);
 
         callback->cv = SvREFCNT_inc(data);
         storeTHX(callback->perl);
-        // warn("here at %s line %d", __FILE__, __LINE__);
+        // warn("callback->sig_len == %d at %s line %d", callback->sig_len, __FILE__, __LINE__);
         cb = dcbNewCallback(callback->sig, cbHandler, callback);
         // warn("here at %s line %d", __FILE__, __LINE__);
         {
@@ -1000,6 +1066,7 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
     } break;
     default: {
         char *str = SvPVbytex_nolen(type);
+        sv_dump(type);
         croak("%c is not a known type in sv2ptr(...)", str[0]);
     }
         //~ warn("here at %s line %d", __FILE__, __LINE__);
